/*
Qt Threading Test Application
Copyright (C) 2005-2014, James Gibbons
Released under the GNU Lesser General Public License version 2.1
See http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
*/
#ifndef QT_THREADTEST_H
#define QT_THREADTEST_H

#include <QtCore>
#include <QtGui>
#include <QMainWindow>

namespace Ui {
	class Qt_ThreadTestForm;
}

class ThreadObject;

class ThreadWorkObject:public QObject
{
	Q_OBJECT
public:
	ThreadWorkObject(QWidget *parent = 0, ThreadObject *setThread=0,
									 const int setID=0);
	~ThreadWorkObject(void);

	const int ourID;					// our ID
	ThreadObject *ourThread;	// thread object that created us

	int runCount;

public slots:
	void countValue(void);
};

class ThreadObject:public QThread
{
	Q_OBJECT
public:
	ThreadObject(QWidget *parent = 0);
	~ThreadObject(void);

	// this lock is used for the messageQueue and threadList
	QMutex lockLists;

	// list of messages from the thread work objects
	// index by ID
	QVector<QString> messageList;

	// a list of the thread work objects
	// index by ID
	QVector<ThreadWorkObject*> threadList;

	QThread *guiThread;						// the GUI thread
	QThread *runThread;						// this thread

	// signaled from GUI timer
	ThreadWorkObject *gT_gO;			// ID=0, GUI object
	ThreadWorkObject *gT_tO;			// ID=1, thread object
	ThreadWorkObject *gT_rO;			// ID=2, QThread::run() object

	// signaled from thread timer
	ThreadWorkObject *tT_gO;			// ID=3, GUI object
	ThreadWorkObject *tT_tO;			// ID=4, thread object
	ThreadWorkObject *tT_rO;			// ID=5, QThread::run() object

	// signaled from QThread::run() timer
	ThreadWorkObject *rT_gO;			// ID=6, GUI object
	ThreadWorkObject *rT_tO;			// ID=7, thread object
	ThreadWorkObject *rT_rO;			// ID=8, QThread::run() object

	QTimer *guiTimer;							// timer not moved to thread (GUI based)
	QTimer *threadTimer;					// timer moved to thread
	QTimer *runTimer;							// timer created on QThread::run()

	void run(void);
};

class Qt_ThreadTest : public QMainWindow
{
	Q_OBJECT
public:
	explicit Qt_ThreadTest(QWidget *parent = 0);
	~Qt_ThreadTest();

private:
	Ui::Qt_ThreadTestForm *ui;

	QTimer guiDisplayTimer;
	ThreadObject testSignalThread;

public slots:
	void runDisplay(void);
signals:
	// send signal to SlotObject
	void sigObject(void);
};

#endif // QT_THREADTEST_H
